@@ -70,7 +70,7 @@
 
 /* Copy the first part of user declarations.  */
 /* Line 358 of yacc.c  */
-#line 1 "i386_parse.y"
+#line 1 "/home/mark/src/elfutils-clean/libcpu/i386_parse.y"
 
 /* Parser for i386 CPU description.
    Copyright (C) 2004, 2005, 2007, 2008, 2009 Red Hat, Inc.
@@ -353,7 +353,7 @@ extern int i386_debug;
 typedef union YYSTYPE
 {
 /* Line 374 of yacc.c  */
-#line 217 "i386_parse.y"
+#line 217 "/home/mark/src/elfutils-clean/libcpu/i386_parse.y"
 
   unsigned long int num;
   char *str;
@@ -1648,7 +1648,7 @@ yyreduce:
     {
         case 2:
 /* Line 1813 of yacc.c  */
-#line 248 "i386_parse.y"
+#line 248 "/home/mark/src/elfutils-clean/libcpu/i386_parse.y"
     {
 		      if (error_message_count != 0)
 			error (EXIT_FAILURE, 0,
@@ -1660,25 +1660,25 @@ yyreduce:
 
   case 5:
 /* Line 1813 of yacc.c  */
-#line 262 "i386_parse.y"
+#line 262 "/home/mark/src/elfutils-clean/libcpu/i386_parse.y"
     { new_bitfield ((yyvsp[(2) - (3)].str), (yyvsp[(3) - (3)].num)); }
     break;
 
   case 6:
 /* Line 1813 of yacc.c  */
-#line 264 "i386_parse.y"
+#line 264 "/home/mark/src/elfutils-clean/libcpu/i386_parse.y"
     { new_bitfield ((yyvsp[(2) - (2)].str), -1); }
     break;
 
   case 7:
 /* Line 1813 of yacc.c  */
-#line 266 "i386_parse.y"
+#line 266 "/home/mark/src/elfutils-clean/libcpu/i386_parse.y"
     { new_bitfield ((yyvsp[(2) - (2)].str), -2); }
     break;
 
   case 8:
 /* Line 1813 of yacc.c  */
-#line 268 "i386_parse.y"
+#line 268 "/home/mark/src/elfutils-clean/libcpu/i386_parse.y"
     {
 		      struct synonym *newp = xmalloc (sizeof (*newp));
 		      newp->from = (yyvsp[(2) - (3)].str);
@@ -1694,7 +1694,7 @@ yyreduce:
 
   case 12:
 /* Line 1813 of yacc.c  */
-#line 287 "i386_parse.y"
+#line 287 "/home/mark/src/elfutils-clean/libcpu/i386_parse.y"
     {
 		      if ((yyvsp[(3) - (6)].field) != NULL && strcmp ((yyvsp[(3) - (6)].field)->name, "RE") != 0
 			  && strcmp ((yyvsp[(3) - (6)].field)->name, "R") != 0)
@@ -1781,7 +1781,7 @@ yyreduce:
 
   case 14:
 /* Line 1813 of yacc.c  */
-#line 373 "i386_parse.y"
+#line 373 "/home/mark/src/elfutils-clean/libcpu/i386_parse.y"
     {
 		      struct known_bitfield search;
 		      search.name = (yyvsp[(1) - (1)].str);
@@ -1800,13 +1800,13 @@ yyreduce:
 
   case 15:
 /* Line 1813 of yacc.c  */
-#line 388 "i386_parse.y"
+#line 388 "/home/mark/src/elfutils-clean/libcpu/i386_parse.y"
     { (yyval.field) = NULL; }
     break;
 
   case 16:
 /* Line 1813 of yacc.c  */
-#line 392 "i386_parse.y"
+#line 392 "/home/mark/src/elfutils-clean/libcpu/i386_parse.y"
     {
 		      check_bits ((yyvsp[(3) - (3)].bit));
 
@@ -1820,7 +1820,7 @@ yyreduce:
 
   case 17:
 /* Line 1813 of yacc.c  */
-#line 402 "i386_parse.y"
+#line 402 "/home/mark/src/elfutils-clean/libcpu/i386_parse.y"
     {
 		      check_bits ((yyvsp[(1) - (1)].bit));
 		      (yyval.bit) = (yyvsp[(1) - (1)].bit);
@@ -1829,7 +1829,7 @@ yyreduce:
 
   case 18:
 /* Line 1813 of yacc.c  */
-#line 409 "i386_parse.y"
+#line 409 "/home/mark/src/elfutils-clean/libcpu/i386_parse.y"
     {
 		      struct bitvalue *runp = (yyvsp[(1) - (2)].bit);
 		      while (runp->next != NULL)
@@ -1841,13 +1841,13 @@ yyreduce:
 
   case 19:
 /* Line 1813 of yacc.c  */
-#line 417 "i386_parse.y"
+#line 417 "/home/mark/src/elfutils-clean/libcpu/i386_parse.y"
     { (yyval.bit) = (yyvsp[(1) - (1)].bit); }
     break;
 
   case 20:
 /* Line 1813 of yacc.c  */
-#line 421 "i386_parse.y"
+#line 421 "/home/mark/src/elfutils-clean/libcpu/i386_parse.y"
     {
 		      (yyval.bit) = xmalloc (sizeof (struct bitvalue));
 		      (yyval.bit)->type = zeroone;
@@ -1858,7 +1858,7 @@ yyreduce:
 
   case 21:
 /* Line 1813 of yacc.c  */
-#line 428 "i386_parse.y"
+#line 428 "/home/mark/src/elfutils-clean/libcpu/i386_parse.y"
     {
 		      (yyval.bit) = xmalloc (sizeof (struct bitvalue));
 		      (yyval.bit)->type = zeroone;
@@ -1869,7 +1869,7 @@ yyreduce:
 
   case 22:
 /* Line 1813 of yacc.c  */
-#line 435 "i386_parse.y"
+#line 435 "/home/mark/src/elfutils-clean/libcpu/i386_parse.y"
     {
 		      (yyval.bit) = xmalloc (sizeof (struct bitvalue));
 		      struct known_bitfield search;
@@ -1893,19 +1893,19 @@ yyreduce:
 
   case 23:
 /* Line 1813 of yacc.c  */
-#line 457 "i386_parse.y"
+#line 457 "/home/mark/src/elfutils-clean/libcpu/i386_parse.y"
     { (yyval.arg) = (yyvsp[(2) - (2)].arg); }
     break;
 
   case 24:
 /* Line 1813 of yacc.c  */
-#line 459 "i386_parse.y"
+#line 459 "/home/mark/src/elfutils-clean/libcpu/i386_parse.y"
     { (yyval.arg) = NULL; }
     break;
 
   case 25:
 /* Line 1813 of yacc.c  */
-#line 463 "i386_parse.y"
+#line 463 "/home/mark/src/elfutils-clean/libcpu/i386_parse.y"
     {
 		      struct argument *runp = (yyvsp[(1) - (3)].arg);
 		      while (runp->next != NULL)
@@ -1919,7 +1919,7 @@ yyreduce:
 
   case 26:
 /* Line 1813 of yacc.c  */
-#line 473 "i386_parse.y"
+#line 473 "/home/mark/src/elfutils-clean/libcpu/i386_parse.y"
     {
 		      (yyval.arg) = xmalloc (sizeof (struct argument));
 		      (yyval.arg)->name = combine ((yyvsp[(1) - (1)].name));
@@ -1929,7 +1929,7 @@ yyreduce:
 
   case 27:
 /* Line 1813 of yacc.c  */
-#line 481 "i386_parse.y"
+#line 481 "/home/mark/src/elfutils-clean/libcpu/i386_parse.y"
     {
 		      struct argname *runp = (yyvsp[(1) - (2)].name);
 		      while (runp->next != NULL)
@@ -1941,13 +1941,13 @@ yyreduce:
 
   case 28:
 /* Line 1813 of yacc.c  */
-#line 489 "i386_parse.y"
+#line 489 "/home/mark/src/elfutils-clean/libcpu/i386_parse.y"
     { (yyval.name) = (yyvsp[(1) - (1)].name); }
     break;
 
   case 29:
 /* Line 1813 of yacc.c  */
-#line 492 "i386_parse.y"
+#line 492 "/home/mark/src/elfutils-clean/libcpu/i386_parse.y"
     {
 		      (yyval.name) = xmalloc (sizeof (struct argname));
 		      (yyval.name)->type = nfield;
@@ -1983,7 +1983,7 @@ yyreduce:
 
   case 30:
 /* Line 1813 of yacc.c  */
-#line 524 "i386_parse.y"
+#line 524 "/home/mark/src/elfutils-clean/libcpu/i386_parse.y"
     {
 		      (yyval.name) = xmalloc (sizeof (struct argname));
 		      (yyval.name)->type = string;
@@ -1996,7 +1996,7 @@ yyreduce:
 
   case 31:
 /* Line 1813 of yacc.c  */
-#line 533 "i386_parse.y"
+#line 533 "/home/mark/src/elfutils-clean/libcpu/i386_parse.y"
     {
 		      (yyval.name) = xmalloc (sizeof (struct argname));
 		      (yyval.name)->type = string;
@@ -2007,7 +2007,7 @@ yyreduce:
 
   case 32:
 /* Line 1813 of yacc.c  */
-#line 540 "i386_parse.y"
+#line 540 "/home/mark/src/elfutils-clean/libcpu/i386_parse.y"
     {
 		      (yyval.name) = xmalloc (sizeof (struct argname));
 		      (yyval.name)->type = string;
@@ -2252,7 +2252,7 @@ yyreturn:
 
 
 /* Line 2076 of yacc.c  */
-#line 550 "i386_parse.y"
+#line 550 "/home/mark/src/elfutils-clean/libcpu/i386_parse.y"
 
 
 static void
