@@ -1,5 +1,5 @@
 /* Standard argp argument parsers for tools using libdwfl.
-   Copyright (C) 2005-2010 Red Hat, Inc.
+   Copyright (C) 2005-2010, 2012 Red Hat, Inc.
    This file is part of elfutils.
 
    This file is free software; you can redistribute it and/or modify
@@ -60,6 +60,32 @@ static const struct argp_option options[] =
   { NULL, 0, NULL, 0, NULL, 0 }
 };
 
+/* Wrapper to provide proper FILE_NAME for -e|--executable.  */
+static int
+offline_find_elf (Dwfl_Module *mod, void **userdata, const char *modname,
+		  Dwarf_Addr base, char **file_name, Elf **elfp)
+{
+  if (modname != NULL && (strcmp (modname, "[exe]") == 0
+			  || strcmp (modname, "[pie]") == 0)
+      && mod->dwfl->executable_for_core)
+    {
+      /* When both --core and --executable are given in whatever order
+	 dwfl_core_file_report is called first and this callback will replace
+	 the Dwfl_Module main.name with the recorded --executable file when the
+	 modname is [exe] or [pie] (which then triggers opening and reporting
+	 of the executable).  */
+      char *e_dup = strdup (mod->dwfl->executable_for_core);
+      if (e_dup)
+	{
+	  free (*file_name);
+	  *file_name = e_dup;
+	  return -1;
+	}
+    }
+  return INTUSE(dwfl_build_id_find_elf) (mod, userdata, modname, base,
+                                         file_name, elfp);
+}
+
 static char *debuginfo_path;
 
 static const Dwfl_Callbacks offline_callbacks =
@@ -70,7 +96,7 @@ static const Dwfl_Callbacks offline_callbacks =
     .section_address = INTUSE(dwfl_offline_section_address),
 
     /* We use this table for core files too.  */
-    .find_elf = INTUSE(dwfl_build_id_find_elf),
+    .find_elf = offline_find_elf,
   };
 
 static const Dwfl_Callbacks proc_callbacks =
@@ -90,6 +116,16 @@ static const Dwfl_Callbacks kernel_callbacks =
     .section_address = INTUSE(dwfl_linux_kernel_module_section_address),
   };
 
+/* Structure held at state->HOOK.  */
+struct parse_opt
+{
+  Dwfl *dwfl;
+  /* The -e|--executable parameter.  */
+  const char *e;
+  /* The --core parameter.  */
+  const char *core;
+};
+
 static error_t
 parse_opt (int key, char *arg, struct argp_state *state)
 {
@@ -111,152 +147,141 @@ parse_opt (int key, char *arg, struct argp_state *state)
 
   switch (key)
     {
+    case ARGP_KEY_INIT:
+      {
+	assert (state->hook == NULL);
+	struct parse_opt *opt = calloc (1, sizeof (*opt));
+	if (opt == NULL)
+	  failure (NULL, DWFL_E_ERRNO, "calloc");
+	state->hook = opt;
+      }
+      break;
+
     case OPT_DEBUGINFO:
       debuginfo_path = arg;
       break;
 
     case 'e':
       {
-	Dwfl *dwfl = state->hook;
+	struct parse_opt *opt = state->hook;
+	Dwfl *dwfl = opt->dwfl;
 	if (dwfl == NULL)
 	  {
 	    dwfl = INTUSE(dwfl_begin) (&offline_callbacks);
 	    if (dwfl == NULL)
 	      return fail (dwfl, -1, arg);
-	    state->hook = dwfl;
+	    opt->dwfl = dwfl;
 
 	    /* Start at zero so if there is just one -e foo.so,
 	       the DSO is shown without address bias.  */
 	    dwfl->offline_next_address = 0;
 	  }
-	if (dwfl->callbacks == &offline_callbacks)
-	  {
-	    if (INTUSE(dwfl_report_offline) (dwfl, "", arg, -1) == NULL)
-	      return fail (dwfl, -1, arg);
-	    state->hook = dwfl;
-	  }
-	else
+	if (dwfl->callbacks != &offline_callbacks)
 	  {
 	  toomany:
 	    argp_error (state, "%s",
 			_("only one of -e, -p, -k, -K, or --core allowed"));
 	    return EINVAL;
 	  }
+	opt->e = arg;
       }
       break;
 
     case 'p':
-      if (state->hook == NULL)
-	{
-	  Dwfl *dwfl = INTUSE(dwfl_begin) (&proc_callbacks);
-	  int result = INTUSE(dwfl_linux_proc_report) (dwfl, atoi (arg));
-	  if (result != 0)
-	    return fail (dwfl, result, arg);
-	  state->hook = dwfl;
-	}
-      else
-	goto toomany;
+      {
+	struct parse_opt *opt = state->hook;
+	if (opt->dwfl == NULL)
+	  {
+	    Dwfl *dwfl = INTUSE(dwfl_begin) (&proc_callbacks);
+	    int result = INTUSE(dwfl_linux_proc_report) (dwfl, atoi (arg));
+	    if (result != 0)
+	      return fail (dwfl, result, arg);
+	    opt->dwfl = dwfl;
+	  }
+	else
+	  goto toomany;
+      }
       break;
 
     case 'M':
-      if (state->hook == NULL)
-	{
-	  FILE *f = fopen (arg, "r");
-	  if (f == NULL)
-	  nofile:
-	    {
-	      int code = errno;
-	      argp_failure (state, EXIT_FAILURE, code,
-			    "cannot open '%s'", arg);
-	      return code;
-	    }
-	  Dwfl *dwfl = INTUSE(dwfl_begin) (&proc_callbacks);
-	  int result = INTUSE(dwfl_linux_proc_maps_report) (dwfl, f);
-	  fclose (f);
-	  if (result != 0)
-	    return fail (dwfl, result, arg);
-	  state->hook = dwfl;
-	}
-      else
-	goto toomany;
+      {
+	struct parse_opt *opt = state->hook;
+	if (opt->dwfl == NULL)
+	  {
+	    FILE *f = fopen (arg, "r");
+	    if (f == NULL)
+	      {
+		int code = errno;
+		argp_failure (state, EXIT_FAILURE, code,
+			      "cannot open '%s'", arg);
+		return code;
+	      }
+	    Dwfl *dwfl = INTUSE(dwfl_begin) (&proc_callbacks);
+	    int result = INTUSE(dwfl_linux_proc_maps_report) (dwfl, f);
+	    fclose (f);
+	    if (result != 0)
+	      return fail (dwfl, result, arg);
+	    opt->dwfl = dwfl;
+	  }
+	else
+	  goto toomany;
+      }
       break;
 
     case OPT_COREFILE:
       {
-	Dwfl *dwfl = state->hook;
+	struct parse_opt *opt = state->hook;
+	Dwfl *dwfl = opt->dwfl;
 	if (dwfl == NULL)
-	  state->hook = dwfl = INTUSE(dwfl_begin) (&offline_callbacks);
+	  opt->dwfl = dwfl = INTUSE(dwfl_begin) (&offline_callbacks);
 	/* Permit -e and --core together.  */
 	else if (dwfl->callbacks != &offline_callbacks)
 	  goto toomany;
-
-	int fd = open64 (arg, O_RDONLY);
-	if (fd < 0)
-	  goto nofile;
-
-	Elf *core;
-	Dwfl_Error error = __libdw_open_file (&fd, &core, true, false);
-	if (error != DWFL_E_NOERROR)
-	  {
-	    argp_failure (state, EXIT_FAILURE, 0,
-			  _("cannot read ELF core file: %s"),
-			  INTUSE(dwfl_errmsg) (error));
-	    return error == DWFL_E_ERRNO ? errno : EIO;
-	  }
-
-	int result = INTUSE(dwfl_core_file_report) (dwfl, core);
-	if (result < 0)
-	  {
-	    elf_end (core);
-	    close (fd);
-	    return fail (dwfl, result, arg);
-	  }
-
-	/* From now we leak FD and CORE.  */
-
-	if (result == 0)
-	  {
-	    argp_failure (state, EXIT_FAILURE, 0,
-			  _("No modules recognized in core file"));
-	    return ENOENT;
-	  }
+	opt->core = arg;
       }
       break;
 
     case 'k':
-      if (state->hook == NULL)
-	{
-	  Dwfl *dwfl = INTUSE(dwfl_begin) (&kernel_callbacks);
-	  int result = INTUSE(dwfl_linux_kernel_report_kernel) (dwfl);
-	  if (result != 0)
-	    return fail (dwfl, result, _("cannot load kernel symbols"));
-	  result = INTUSE(dwfl_linux_kernel_report_modules) (dwfl);
-	  if (result != 0)
-	    /* Non-fatal to have no modules since we do have the kernel.  */
-	    failure (dwfl, result, _("cannot find kernel modules"));
-	  state->hook = dwfl;
-	}
-      else
-	goto toomany;
+      {
+	struct parse_opt *opt = state->hook;
+	if (opt->dwfl == NULL)
+	  {
+	    Dwfl *dwfl = INTUSE(dwfl_begin) (&kernel_callbacks);
+	    int result = INTUSE(dwfl_linux_kernel_report_kernel) (dwfl);
+	    if (result != 0)
+	      return fail (dwfl, result, _("cannot load kernel symbols"));
+	    result = INTUSE(dwfl_linux_kernel_report_modules) (dwfl);
+	    if (result != 0)
+	      /* Non-fatal to have no modules since we do have the kernel.  */
+	      failure (dwfl, result, _("cannot find kernel modules"));
+	    opt->dwfl = dwfl;
+	  }
+	else
+	  goto toomany;
+      }
       break;
 
     case 'K':
-      if (state->hook == NULL)
-	{
-	  Dwfl *dwfl = INTUSE(dwfl_begin) (&offline_callbacks);
-	  int result = INTUSE(dwfl_linux_kernel_report_offline) (dwfl, arg,
-								 NULL);
-	  if (result != 0)
-	    return fail (dwfl, result, _("cannot find kernel or modules"));
-	  state->hook = dwfl;
-	}
-      else
-	goto toomany;
+      {
+	struct parse_opt *opt = state->hook;
+	if (opt->dwfl == NULL)
+	  {
+	    Dwfl *dwfl = INTUSE(dwfl_begin) (&offline_callbacks);
+	    int result = INTUSE(dwfl_linux_kernel_report_offline) (dwfl, arg,
+								   NULL);
+	    if (result != 0)
+	      return fail (dwfl, result, _("cannot find kernel or modules"));
+	    opt->dwfl = dwfl;
+	  }
+	else
+	  goto toomany;
+      }
       break;
 
     case ARGP_KEY_SUCCESS:
       {
-	Dwfl *dwfl = state->hook;
+	struct parse_opt *opt = state->hook;
+	Dwfl *dwfl = opt->dwfl;
 
 	if (dwfl == NULL)
 	  {
@@ -265,7 +290,54 @@ parse_opt (int key, char *arg, struct argp_state *state)
 	    dwfl = INTUSE(dwfl_begin) (&offline_callbacks);
 	    if (INTUSE(dwfl_report_offline) (dwfl, "", arg, -1) == NULL)
 	      return fail (dwfl, -1, arg);
-	    state->hook = dwfl;
+	    opt->dwfl = dwfl;
+	  }
+
+	if (opt->core)
+	  {
+	    if (opt->e)
+	      dwfl->executable_for_core = strdup (opt->e);
+
+	    int fd = open64 (opt->core, O_RDONLY);
+	    if (fd < 0)
+	      {
+		int code = errno;
+		argp_failure (state, EXIT_FAILURE, code,
+			      "cannot open '%s'", opt->core);
+		return code;
+	      }
+
+	    Elf *core;
+	    Dwfl_Error error = __libdw_open_file (&fd, &core, true, false);
+	    if (error != DWFL_E_NOERROR)
+	      {
+		argp_failure (state, EXIT_FAILURE, 0,
+			      _("cannot read ELF core file: %s"),
+			      INTUSE(dwfl_errmsg) (error));
+		return error == DWFL_E_ERRNO ? errno : EIO;
+	      }
+
+	    int result = INTUSE(dwfl_core_file_report) (dwfl, core);
+	    if (result < 0)
+	      {
+		elf_end (core);
+		close (fd);
+		return fail (dwfl, result, opt->core);
+	      }
+
+	    /* From now we leak FD and CORE.  */
+
+	    if (result == 0)
+	      {
+		argp_failure (state, EXIT_FAILURE, 0,
+			      _("No modules recognized in core file"));
+		return ENOENT;
+	      }
+	  }
+	else if (opt->e)
+	  {
+	    if (INTUSE(dwfl_report_offline) (dwfl, "", opt->e, -1) == NULL)
+	      return fail (dwfl, -1, opt->e);
 	  }
 
 	/* One of the three flavors has done dwfl_begin and some reporting
@@ -274,12 +346,22 @@ parse_opt (int key, char *arg, struct argp_state *state)
 
 	int result = INTUSE(dwfl_report_end) (dwfl, NULL, NULL);
 	assert (result == 0);
+
+	/* Update the input all along, so a parent parser can see it.
+	   As we free OPT the update below will be no longer active.  */
+	*(Dwfl **) state->input = dwfl;
+	free (opt);
+	state->hook = NULL;
       }
       break;
 
     case ARGP_KEY_ERROR:
-      dwfl_end (state->hook);
-      state->hook = NULL;
+      {
+	struct parse_opt *opt = state->hook;
+	dwfl_end (opt->dwfl);
+	free (opt);
+	state->hook = NULL;
+      }
       break;
 
     default:
@@ -287,7 +369,10 @@ parse_opt (int key, char *arg, struct argp_state *state)
     }
 
   /* Update the input all along, so a parent parser can see it.  */
-  *(Dwfl **) state->input = state->hook;
+  struct parse_opt *opt = state->hook;
+  if (opt)
+    *(Dwfl **) state->input = opt->dwfl;
+
   return 0;
 }
 
