@@ -70,7 +70,7 @@
 
 /* Copy the first part of user declarations.  */
 /* Line 358 of yacc.c  */
-#line 1 "ldscript.y"
+#line 1 "/home/mark/src/elfutils-clean/src/ldscript.y"
 
 /* Parser for linker scripts.
    Copyright (C) 2001-2011 Red Hat, Inc.
@@ -233,7 +233,7 @@ extern int lddebug;
 typedef union YYSTYPE
 {
 /* Line 374 of yacc.c  */
-#line 63 "ldscript.y"
+#line 63 "/home/mark/src/elfutils-clean/src/ldscript.y"
 
   uintmax_t num;
   enum expression_tag op;
@@ -1645,13 +1645,13 @@ yyreduce:
     {
         case 3:
 /* Line 1813 of yacc.c  */
-#line 137 "ldscript.y"
+#line 137 "/home/mark/src/elfutils-clean/src/ldscript.y"
     { add_versions ((yyvsp[(2) - (2)].version)); }
     break;
 
   case 6:
 /* Line 1813 of yacc.c  */
-#line 145 "ldscript.y"
+#line 145 "/home/mark/src/elfutils-clean/src/ldscript.y"
     {
 		      if (likely (ld_state.entry == NULL))
 			ld_state.entry = (yyvsp[(3) - (5)].str);
@@ -1660,7 +1660,7 @@ yyreduce:
 
   case 7:
 /* Line 1813 of yacc.c  */
-#line 150 "ldscript.y"
+#line 150 "/home/mark/src/elfutils-clean/src/ldscript.y"
     {
 		      ld_new_searchdir ((yyvsp[(3) - (5)].str));
 		    }
@@ -1668,7 +1668,7 @@ yyreduce:
 
   case 8:
 /* Line 1813 of yacc.c  */
-#line 154 "ldscript.y"
+#line 154 "/home/mark/src/elfutils-clean/src/ldscript.y"
     {
 		      if (likely (ld_state.pagesize == 0))
 			ld_state.pagesize = (yyvsp[(3) - (5)].num);
@@ -1677,7 +1677,7 @@ yyreduce:
 
   case 9:
 /* Line 1813 of yacc.c  */
-#line 159 "ldscript.y"
+#line 159 "/home/mark/src/elfutils-clean/src/ldscript.y"
     {
 		      if (likely (ld_state.interp == NULL)
 			  && ld_state.file_type != dso_file_type)
@@ -1687,7 +1687,7 @@ yyreduce:
 
   case 10:
 /* Line 1813 of yacc.c  */
-#line 165 "ldscript.y"
+#line 165 "/home/mark/src/elfutils-clean/src/ldscript.y"
     {
 		      new_segment ((yyvsp[(2) - (5)].num), (yyvsp[(4) - (5)].output_rule));
 		    }
@@ -1695,7 +1695,7 @@ yyreduce:
 
   case 11:
 /* Line 1813 of yacc.c  */
-#line 169 "ldscript.y"
+#line 169 "/home/mark/src/elfutils-clean/src/ldscript.y"
     {
 		      fputs_unlocked (gettext ("mode for segment invalid\n"),
 				      stderr);
@@ -1705,7 +1705,7 @@ yyreduce:
 
   case 12:
 /* Line 1813 of yacc.c  */
-#line 175 "ldscript.y"
+#line 175 "/home/mark/src/elfutils-clean/src/ldscript.y"
     {
 		      /* First little optimization.  If there is only one
 			 file in the group don't do anything.  */
@@ -1720,31 +1720,31 @@ yyreduce:
 
   case 13:
 /* Line 1813 of yacc.c  */
-#line 186 "ldscript.y"
+#line 186 "/home/mark/src/elfutils-clean/src/ldscript.y"
     { add_inputfiles ((yyvsp[(3) - (4)].filename_list)); }
     break;
 
   case 14:
 /* Line 1813 of yacc.c  */
-#line 188 "ldscript.y"
+#line 188 "/home/mark/src/elfutils-clean/src/ldscript.y"
     { add_inputfiles (mark_as_needed ((yyvsp[(3) - (4)].filename_list))); }
     break;
 
   case 15:
 /* Line 1813 of yacc.c  */
-#line 190 "ldscript.y"
+#line 190 "/home/mark/src/elfutils-clean/src/ldscript.y"
     { add_versions ((yyvsp[(3) - (4)].version)); }
     break;
 
   case 16:
 /* Line 1813 of yacc.c  */
-#line 192 "ldscript.y"
+#line 192 "/home/mark/src/elfutils-clean/src/ldscript.y"
     { /* XXX TODO */ }
     break;
 
   case 17:
 /* Line 1813 of yacc.c  */
-#line 196 "ldscript.y"
+#line 196 "/home/mark/src/elfutils-clean/src/ldscript.y"
     {
 		      (yyvsp[(2) - (2)].output_rule)->next = (yyvsp[(1) - (2)].output_rule)->next;
 		      (yyval.output_rule) = (yyvsp[(1) - (2)].output_rule)->next = (yyvsp[(2) - (2)].output_rule);
@@ -1753,13 +1753,13 @@ yyreduce:
 
   case 18:
 /* Line 1813 of yacc.c  */
-#line 201 "ldscript.y"
+#line 201 "/home/mark/src/elfutils-clean/src/ldscript.y"
     { (yyval.output_rule) = (yyvsp[(1) - (1)].output_rule); }
     break;
 
   case 19:
 /* Line 1813 of yacc.c  */
-#line 205 "ldscript.y"
+#line 205 "/home/mark/src/elfutils-clean/src/ldscript.y"
     {
 		      (yyval.output_rule) = new_output_rule (output_assignment);
 		      (yyval.output_rule)->val.assignment = (yyvsp[(1) - (2)].assignment);
@@ -1768,7 +1768,7 @@ yyreduce:
 
   case 20:
 /* Line 1813 of yacc.c  */
-#line 210 "ldscript.y"
+#line 210 "/home/mark/src/elfutils-clean/src/ldscript.y"
     {
 		      (yyval.output_rule) = new_output_rule (output_section);
 		      (yyval.output_rule)->val.section.name = (yyvsp[(1) - (4)].str);
@@ -1784,7 +1784,7 @@ yyreduce:
 
   case 21:
 /* Line 1813 of yacc.c  */
-#line 222 "ldscript.y"
+#line 222 "/home/mark/src/elfutils-clean/src/ldscript.y"
     {
 		      /* This is a short cut for "ID { *(ID) }".  */
 		      (yyval.output_rule) = new_output_rule (output_section);
@@ -1810,19 +1810,19 @@ yyreduce:
 
   case 22:
 /* Line 1813 of yacc.c  */
-#line 246 "ldscript.y"
+#line 246 "/home/mark/src/elfutils-clean/src/ldscript.y"
     { (yyval.assignment) = new_assignment ((yyvsp[(1) - (3)].str), (yyvsp[(3) - (3)].expr), false); }
     break;
 
   case 23:
 /* Line 1813 of yacc.c  */
-#line 248 "ldscript.y"
+#line 248 "/home/mark/src/elfutils-clean/src/ldscript.y"
     { (yyval.assignment) = new_assignment ((yyvsp[(3) - (6)].str), (yyvsp[(5) - (6)].expr), true); }
     break;
 
   case 24:
 /* Line 1813 of yacc.c  */
-#line 252 "ldscript.y"
+#line 252 "/home/mark/src/elfutils-clean/src/ldscript.y"
     {
 		      (yyvsp[(2) - (2)].input_rule)->next = (yyvsp[(1) - (2)].input_rule)->next;
 		      (yyval.input_rule) = (yyvsp[(1) - (2)].input_rule)->next = (yyvsp[(2) - (2)].input_rule);
@@ -1831,13 +1831,13 @@ yyreduce:
 
   case 25:
 /* Line 1813 of yacc.c  */
-#line 257 "ldscript.y"
+#line 257 "/home/mark/src/elfutils-clean/src/ldscript.y"
     { (yyval.input_rule) = (yyvsp[(1) - (1)].input_rule); }
     break;
 
   case 26:
 /* Line 1813 of yacc.c  */
-#line 261 "ldscript.y"
+#line 261 "/home/mark/src/elfutils-clean/src/ldscript.y"
     {
 		      (yyval.input_rule) = new_input_rule (input_section);
 		      (yyval.input_rule)->val.section = (yyvsp[(1) - (1)].filemask_section_name);
@@ -1846,7 +1846,7 @@ yyreduce:
 
   case 27:
 /* Line 1813 of yacc.c  */
-#line 266 "ldscript.y"
+#line 266 "/home/mark/src/elfutils-clean/src/ldscript.y"
     {
 		      (yyvsp[(3) - (4)].filemask_section_name)->keep_flag = true;
 
@@ -1857,7 +1857,7 @@ yyreduce:
 
   case 28:
 /* Line 1813 of yacc.c  */
-#line 273 "ldscript.y"
+#line 273 "/home/mark/src/elfutils-clean/src/ldscript.y"
     {
 		      (yyval.input_rule) = new_input_rule (input_assignment);
 		      (yyval.input_rule)->val.assignment = (yyvsp[(1) - (2)].assignment);
@@ -1866,7 +1866,7 @@ yyreduce:
 
   case 29:
 /* Line 1813 of yacc.c  */
-#line 280 "ldscript.y"
+#line 280 "/home/mark/src/elfutils-clean/src/ldscript.y"
     {
 		      (yyval.filemask_section_name) = (struct filemask_section_name *)
 			obstack_alloc (&ld_state.smem, sizeof (*(yyval.filemask_section_name)));
@@ -1879,31 +1879,31 @@ yyreduce:
 
   case 30:
 /* Line 1813 of yacc.c  */
-#line 291 "ldscript.y"
+#line 291 "/home/mark/src/elfutils-clean/src/ldscript.y"
     { (yyval.sectionname) = new_input_section_name ((yyvsp[(1) - (1)].str), false); }
     break;
 
   case 31:
 /* Line 1813 of yacc.c  */
-#line 293 "ldscript.y"
+#line 293 "/home/mark/src/elfutils-clean/src/ldscript.y"
     { (yyval.sectionname) = new_input_section_name ((yyvsp[(3) - (4)].str), true); }
     break;
 
   case 32:
 /* Line 1813 of yacc.c  */
-#line 297 "ldscript.y"
+#line 297 "/home/mark/src/elfutils-clean/src/ldscript.y"
     { (yyval.str) = (yyvsp[(3) - (4)].str); }
     break;
 
   case 33:
 /* Line 1813 of yacc.c  */
-#line 299 "ldscript.y"
+#line 299 "/home/mark/src/elfutils-clean/src/ldscript.y"
     { (yyval.str) = NULL; }
     break;
 
   case 34:
 /* Line 1813 of yacc.c  */
-#line 303 "ldscript.y"
+#line 303 "/home/mark/src/elfutils-clean/src/ldscript.y"
     {
 		      (yyval.expr) = new_expr (exp_align);
 		      (yyval.expr)->val.child = (yyvsp[(3) - (4)].expr);
@@ -1912,13 +1912,13 @@ yyreduce:
 
   case 35:
 /* Line 1813 of yacc.c  */
-#line 308 "ldscript.y"
+#line 308 "/home/mark/src/elfutils-clean/src/ldscript.y"
     { (yyval.expr) = (yyvsp[(2) - (3)].expr); }
     break;
 
   case 36:
 /* Line 1813 of yacc.c  */
-#line 310 "ldscript.y"
+#line 310 "/home/mark/src/elfutils-clean/src/ldscript.y"
     {
 		      (yyval.expr) = new_expr (exp_mult);
 		      (yyval.expr)->val.binary.left = (yyvsp[(1) - (3)].expr);
@@ -1928,7 +1928,7 @@ yyreduce:
 
   case 37:
 /* Line 1813 of yacc.c  */
-#line 316 "ldscript.y"
+#line 316 "/home/mark/src/elfutils-clean/src/ldscript.y"
     {
 		      (yyval.expr) = new_expr ((yyvsp[(2) - (3)].op));
 		      (yyval.expr)->val.binary.left = (yyvsp[(1) - (3)].expr);
@@ -1938,7 +1938,7 @@ yyreduce:
 
   case 38:
 /* Line 1813 of yacc.c  */
-#line 322 "ldscript.y"
+#line 322 "/home/mark/src/elfutils-clean/src/ldscript.y"
     {
 		      (yyval.expr) = new_expr ((yyvsp[(2) - (3)].op));
 		      (yyval.expr)->val.binary.left = (yyvsp[(1) - (3)].expr);
@@ -1948,7 +1948,7 @@ yyreduce:
 
   case 39:
 /* Line 1813 of yacc.c  */
-#line 328 "ldscript.y"
+#line 328 "/home/mark/src/elfutils-clean/src/ldscript.y"
     {
 		      (yyval.expr) = new_expr (exp_and);
 		      (yyval.expr)->val.binary.left = (yyvsp[(1) - (3)].expr);
@@ -1958,7 +1958,7 @@ yyreduce:
 
   case 40:
 /* Line 1813 of yacc.c  */
-#line 334 "ldscript.y"
+#line 334 "/home/mark/src/elfutils-clean/src/ldscript.y"
     {
 		      (yyval.expr) = new_expr (exp_or);
 		      (yyval.expr)->val.binary.left = (yyvsp[(1) - (3)].expr);
@@ -1968,7 +1968,7 @@ yyreduce:
 
   case 41:
 /* Line 1813 of yacc.c  */
-#line 340 "ldscript.y"
+#line 340 "/home/mark/src/elfutils-clean/src/ldscript.y"
     {
 		      (yyval.expr) = new_expr (exp_num);
 		      (yyval.expr)->val.num = (yyvsp[(1) - (1)].num);
@@ -1977,7 +1977,7 @@ yyreduce:
 
   case 42:
 /* Line 1813 of yacc.c  */
-#line 345 "ldscript.y"
+#line 345 "/home/mark/src/elfutils-clean/src/ldscript.y"
     {
 		      (yyval.expr) = new_expr (exp_id);
 		      (yyval.expr)->val.str = (yyvsp[(1) - (1)].str);
@@ -1986,19 +1986,19 @@ yyreduce:
 
   case 43:
 /* Line 1813 of yacc.c  */
-#line 350 "ldscript.y"
+#line 350 "/home/mark/src/elfutils-clean/src/ldscript.y"
     { (yyval.expr) = new_expr (exp_sizeof_headers); }
     break;
 
   case 44:
 /* Line 1813 of yacc.c  */
-#line 352 "ldscript.y"
+#line 352 "/home/mark/src/elfutils-clean/src/ldscript.y"
     { (yyval.expr) = new_expr (exp_pagesize); }
     break;
 
   case 45:
 /* Line 1813 of yacc.c  */
-#line 356 "ldscript.y"
+#line 356 "/home/mark/src/elfutils-clean/src/ldscript.y"
     {
 		      (yyvsp[(3) - (3)].filename_list)->next = (yyvsp[(1) - (3)].filename_list)->next;
 		      (yyval.filename_list) = (yyvsp[(1) - (3)].filename_list)->next = (yyvsp[(3) - (3)].filename_list);
@@ -2007,13 +2007,13 @@ yyreduce:
 
   case 46:
 /* Line 1813 of yacc.c  */
-#line 361 "ldscript.y"
+#line 361 "/home/mark/src/elfutils-clean/src/ldscript.y"
     { (yyval.filename_list) = (yyvsp[(1) - (1)].filename_list); }
     break;
 
   case 49:
 /* Line 1813 of yacc.c  */
-#line 369 "ldscript.y"
+#line 369 "/home/mark/src/elfutils-clean/src/ldscript.y"
     {
 		      /* First little optimization.  If there is only one
 			 file in the group don't do anything.  */
@@ -2028,19 +2028,19 @@ yyreduce:
 
   case 50:
 /* Line 1813 of yacc.c  */
-#line 380 "ldscript.y"
+#line 380 "/home/mark/src/elfutils-clean/src/ldscript.y"
     { (yyval.filename_list) = mark_as_needed ((yyvsp[(3) - (4)].filename_list)); }
     break;
 
   case 51:
 /* Line 1813 of yacc.c  */
-#line 382 "ldscript.y"
+#line 382 "/home/mark/src/elfutils-clean/src/ldscript.y"
     { (yyval.filename_list) = new_filename_listelem ((yyvsp[(1) - (1)].str)); }
     break;
 
   case 52:
 /* Line 1813 of yacc.c  */
-#line 387 "ldscript.y"
+#line 387 "/home/mark/src/elfutils-clean/src/ldscript.y"
     {
 		      (yyvsp[(2) - (2)].version)->next = (yyvsp[(1) - (2)].version)->next;
 		      (yyval.version) = (yyvsp[(1) - (2)].version)->next = (yyvsp[(2) - (2)].version);
@@ -2049,13 +2049,13 @@ yyreduce:
 
   case 53:
 /* Line 1813 of yacc.c  */
-#line 392 "ldscript.y"
+#line 392 "/home/mark/src/elfutils-clean/src/ldscript.y"
     { (yyval.version) = (yyvsp[(1) - (1)].version); }
     break;
 
   case 54:
 /* Line 1813 of yacc.c  */
-#line 396 "ldscript.y"
+#line 396 "/home/mark/src/elfutils-clean/src/ldscript.y"
     {
 		      (yyvsp[(2) - (4)].version)->versionname = "";
 		      (yyvsp[(2) - (4)].version)->parentname = NULL;
@@ -2065,7 +2065,7 @@ yyreduce:
 
   case 55:
 /* Line 1813 of yacc.c  */
-#line 402 "ldscript.y"
+#line 402 "/home/mark/src/elfutils-clean/src/ldscript.y"
     {
 		      (yyvsp[(3) - (5)].version)->versionname = (yyvsp[(1) - (5)].str);
 		      (yyvsp[(3) - (5)].version)->parentname = NULL;
@@ -2075,7 +2075,7 @@ yyreduce:
 
   case 56:
 /* Line 1813 of yacc.c  */
-#line 408 "ldscript.y"
+#line 408 "/home/mark/src/elfutils-clean/src/ldscript.y"
     {
 		      (yyvsp[(3) - (6)].version)->versionname = (yyvsp[(1) - (6)].str);
 		      (yyvsp[(3) - (6)].version)->parentname = (yyvsp[(5) - (6)].str);
@@ -2085,31 +2085,31 @@ yyreduce:
 
   case 57:
 /* Line 1813 of yacc.c  */
-#line 417 "ldscript.y"
+#line 417 "/home/mark/src/elfutils-clean/src/ldscript.y"
     { (yyval.version) = merge_versions ((yyvsp[(1) - (2)].version), (yyvsp[(2) - (2)].version)); }
     break;
 
   case 58:
 /* Line 1813 of yacc.c  */
-#line 419 "ldscript.y"
+#line 419 "/home/mark/src/elfutils-clean/src/ldscript.y"
     { (yyval.version) = (yyvsp[(1) - (1)].version); }
     break;
 
   case 59:
 /* Line 1813 of yacc.c  */
-#line 423 "ldscript.y"
+#line 423 "/home/mark/src/elfutils-clean/src/ldscript.y"
     { (yyval.version) = new_version (NULL, (yyvsp[(2) - (2)].id_list)); }
     break;
 
   case 60:
 /* Line 1813 of yacc.c  */
-#line 425 "ldscript.y"
+#line 425 "/home/mark/src/elfutils-clean/src/ldscript.y"
     { (yyval.version) = new_version ((yyvsp[(2) - (2)].id_list), NULL); }
     break;
 
   case 61:
 /* Line 1813 of yacc.c  */
-#line 430 "ldscript.y"
+#line 430 "/home/mark/src/elfutils-clean/src/ldscript.y"
     {
 		      struct id_list *newp = new_id_listelem ((yyvsp[(2) - (3)].str));
 		      newp->next = (yyvsp[(1) - (3)].id_list)->next;
@@ -2119,31 +2119,31 @@ yyreduce:
 
   case 62:
 /* Line 1813 of yacc.c  */
-#line 436 "ldscript.y"
+#line 436 "/home/mark/src/elfutils-clean/src/ldscript.y"
     { (yyval.id_list) = new_id_listelem ((yyvsp[(1) - (2)].str)); }
     break;
 
   case 63:
 /* Line 1813 of yacc.c  */
-#line 440 "ldscript.y"
+#line 440 "/home/mark/src/elfutils-clean/src/ldscript.y"
     { (yyval.str) = (yyvsp[(1) - (1)].str); }
     break;
 
   case 64:
 /* Line 1813 of yacc.c  */
-#line 442 "ldscript.y"
+#line 442 "/home/mark/src/elfutils-clean/src/ldscript.y"
     { (yyval.str) = (yyvsp[(1) - (1)].str); }
     break;
 
   case 65:
 /* Line 1813 of yacc.c  */
-#line 446 "ldscript.y"
+#line 446 "/home/mark/src/elfutils-clean/src/ldscript.y"
     { (yyval.str) = (yyvsp[(1) - (1)].str); }
     break;
 
   case 66:
 /* Line 1813 of yacc.c  */
-#line 448 "ldscript.y"
+#line 448 "/home/mark/src/elfutils-clean/src/ldscript.y"
     { (yyval.str) = NULL; }
     break;
 
@@ -2381,7 +2381,7 @@ yyreturn:
 
 
 /* Line 2076 of yacc.c  */
-#line 451 "ldscript.y"
+#line 451 "/home/mark/src/elfutils-clean/src/ldscript.y"
 
 
 static void
