@@ -110,7 +110,7 @@ extern int lddebug;
 typedef union YYSTYPE
 {
 /* Line 2077 of yacc.c  */
-#line 63 "ldscript.y"
+#line 63 "/home/mark/src/elfutils-clean/src/ldscript.y"
 
   uintmax_t num;
   enum expression_tag op;
